@@ -124,6 +124,14 @@ struct _hash_iter_context_t {
     element_t *p;
 };
 
+enum hash_iter_state {
+    HI_STATE_0,
+    HI_STATE_1,
+    HI_STATE_2,
+    HI_STATE_3A,
+    HI_STATE_3B
+};
+
 /*****************************************************************************/
 /**********************  External Function Declarations  *********************/
 /*****************************************************************************/
@@ -147,7 +155,7 @@ static hash_entry_t *hash_iter_next(struct hash_iter_context_t *iter);
 /*************************  Internal Global Variables  ***********************/
 /*****************************************************************************/
 
-#if DEBUG
+#ifdef DEBUG
 int debug_level = 1;
 #endif
 
@@ -168,7 +176,7 @@ static void sys_free_wrapper(void *ptr, void *pvt)
 static address_t convert_key(hash_key_t *key)
 {
     address_t h;
-    unsigned char *k;
+    const unsigned char *k;
 
     switch(key->type) {
     case HASH_KEY_ULONG:
@@ -176,7 +184,12 @@ static address_t convert_key(hash_key_t *key)
         break;
     case HASH_KEY_STRING:
         /* Convert string to integer */
-        for (h = 0, k = (unsigned char *) key->str; *k; k++)
+        for (h = 0, k = (const unsigned char *) key->str; *k; k++)
+            h = h * PRIME_1 ^ (*k - ' ');
+        break;
+    case HASH_KEY_CONST_STRING:
+        /* Convert string to integer */
+        for (h = 0, k = (const unsigned char *) key->c_str; *k; k++)
             h = h * PRIME_1 ^ (*k - ' ');
         break;
     default:
@@ -204,6 +217,7 @@ static bool is_valid_key_type(hash_key_enum key_type)
     switch(key_type) {
     case HASH_KEY_ULONG:
     case HASH_KEY_STRING:
+    case HASH_KEY_CONST_STRING:
         return true;
     default:
         return false;
@@ -236,6 +250,8 @@ static bool key_equal(hash_key_t *a, hash_key_t *b)
         return (a->ul == b->ul);
     case HASH_KEY_STRING:
         return (strcmp(a->str, b->str) == 0);
+    case HASH_KEY_CONST_STRING:
+        return (strcmp(a->c_str, b->c_str) == 0);
     }
     return false;
 }
@@ -612,7 +628,7 @@ int hash_create_ex(unsigned long count, hash_table_t **tbl,
     table->min_load_factor = min_load_factor == 0 ? HASH_DEFAULT_MIN_LOAD_FACTOR : min_load_factor;
     table->max_load_factor = max_load_factor == 0 ? HASH_DEFAULT_MAX_LOAD_FACTOR : max_load_factor;
 
-#if DEBUG
+#ifdef DEBUG
     if (debug_level >= 1) {
         fprintf(stderr, "hash_create_ex: count=%lu available buckets=%lu bucket_count=%lu maxp=%lu\n",
                 count, table->directory_size*table->segment_size, table->bucket_count, table->maxp);
@@ -662,8 +678,11 @@ int hash_destroy(hash_table_t *table)
                         while (p != NULL) {
                             q = p->next;
                             hdelete_callback(table, HASH_TABLE_DESTROY, &p->entry);
-                            if (p->entry.key.type == HASH_KEY_STRING) {
-                                hfree(table, (char *)p->entry.key.str);
+                            if (p->entry.key.type == HASH_KEY_STRING
+                                    || p->entry.key.type == HASH_KEY_CONST_STRING) {
+                                /* Internally we do not use constant memory for keys
+                                 * in hash table elements. */
+                                hfree(table, p->entry.key.str);
                             }
                             hfree(table, (char *)p);
                             p = q;
@@ -708,33 +727,60 @@ int hash_iterate(hash_table_t *table, hash_iterate_callback callback, void *user
 static hash_entry_t *hash_iter_next(struct hash_iter_context_t *iter_arg)
 {
     struct _hash_iter_context_t *iter = (struct _hash_iter_context_t *) iter_arg;
-    hash_entry_t *entry;
+    hash_entry_t *entry = NULL;
+    enum hash_iter_state state = HI_STATE_3A;
 
     if (iter->table == NULL) return NULL;
-    goto state_3a;
 
- state_1:
-    iter->i++;
-    if(iter->i >= iter->table->segment_count) return NULL;
-    /* test probably unnecessary */
-    iter->s = iter->table->directory[iter->i];
-    if (iter->s == NULL) goto state_1;
-    iter->j = 0;
- state_2:
-    if (iter->j >= iter->table->segment_size) goto state_1;
-    iter->p = iter->s[iter->j];
- state_3a:
-    if (iter->p == NULL) goto state_3b;
-    entry = &iter->p->entry;
-    iter->p = iter->p->next;
+    while (state != HI_STATE_0) {
+
+        switch (state) {
+            case HI_STATE_1:
+                iter->i++;
+                if(iter->i >= iter->table->segment_count) return NULL;
+                /* test probably unnecessary */
+                iter->s = iter->table->directory[iter->i];
+                if (iter->s == NULL) {
+                    state = HI_STATE_1;
+                    break;
+                }
+                iter->j = 0;
+                state = HI_STATE_2;
+                /* FALLTHROUGH */
+
+            case HI_STATE_2:
+                if (iter->j >= iter->table->segment_size) {
+                    state = HI_STATE_1;
+                    break;
+                }
+                iter->p = iter->s[iter->j];
+                state = HI_STATE_3A;
+                /* FALLTHROUGH */
+
+            case HI_STATE_3A:
+                if (iter->p == NULL) {
+                    state = HI_STATE_3B;
+                    break;
+                }
+                entry = &iter->p->entry;
+                iter->p = iter->p->next;
+                state = HI_STATE_0;
+                break;
+
+            case HI_STATE_3B:
+                iter->j++;
+                state = HI_STATE_2;
+                break;
+
+            default:
+                /* Should never reach here */
+                fprintf(stderr, "ERROR hash_iter_next reached invalid state\n");
+                return NULL;
+                break;
+        }
+    }
+
     return entry;
- state_3b:
-    iter->j++;
-    goto state_2;
-
-    /* Should never reach here */
-    fprintf(stderr, "ERROR hash_iter_next reached invalid state\n");
-    return NULL;
 }
 
 struct hash_iter_context_t *new_hash_iter_context(hash_table_t *table)
@@ -768,12 +814,13 @@ unsigned long hash_count(hash_table_t *table)
 
 int hash_keys(hash_table_t *table, unsigned long *count_arg, hash_key_t **keys_arg)
 {
-    unsigned long count = table->entry_count;
+    unsigned long count;
     hash_key_t *keys;
     hash_keys_callback_data_t data;
 
     if (!table) return HASH_ERROR_BAD_TABLE;
 
+    count = table->entry_count;
     if (count == 0) {
         *count_arg = 0;
         *keys_arg = NULL;
@@ -799,12 +846,13 @@ int hash_keys(hash_table_t *table, unsigned long *count_arg, hash_key_t **keys_a
 
 int hash_values(hash_table_t *table, unsigned long *count_arg, hash_value_t **values_arg)
 {
-    unsigned long count = table->entry_count;
+    unsigned long count;
     hash_value_t *values;
     hash_values_callback_data_t data;
 
     if (!table) return HASH_ERROR_BAD_TABLE;
 
+    count = table->entry_count;
     if (count == 0) {
         *count_arg = 0;
         *values_arg = NULL;
@@ -843,12 +891,13 @@ static bool hash_entries_callback(hash_entry_t *item, void *user_data)
 
 int hash_entries(hash_table_t *table, unsigned long *count_arg, hash_entry_t **entries_arg)
 {
-    unsigned long count = table->entry_count;
+    unsigned long count;
     hash_entry_t *entries;
     hash_entries_callback_data_t data;
 
     if (!table) return HASH_ERROR_BAD_TABLE;
 
+    count = table->entry_count;
     if (count == 0) {
         *count_arg = 0;
         *entries_arg = NULL;
@@ -936,13 +985,14 @@ int hash_enter(hash_table_t *table, hash_key_t *key, hash_value_t *value)
             element->entry.key.ul = key->ul;
             break;
         case HASH_KEY_STRING:
-            len = strlen(key->str)+1;
+        case HASH_KEY_CONST_STRING:
+            len = strlen(key->c_str) + 1;
             element->entry.key.str = halloc(table, len);
             if (element->entry.key.str == NULL) {
                 hfree(table, element);
                 return HASH_ERROR_NO_MEMORY;
             }
-            memcpy((void *)element->entry.key.str, key->str, len);
+            memcpy(element->entry.key.str, key->str, len);
             break;
         }
 
@@ -1034,8 +1084,9 @@ int hash_delete(hash_table_t *table, hash_key_t *key)
                 return error;
             }
         }
-        if (element->entry.key.type == HASH_KEY_STRING) {
-            hfree(table, (char *)element->entry.key.str);
+        if (element->entry.key.type == HASH_KEY_STRING
+                || element->entry.key.type == HASH_KEY_CONST_STRING) {
+            hfree(table, element->entry.key.str);
         }
         hfree(table, element);
         return HASH_SUCCESS;
