@@ -112,7 +112,8 @@ typedef struct hash_table_str hash_table_t;
 
 typedef enum {
     HASH_KEY_STRING,
-    HASH_KEY_ULONG
+    HASH_KEY_ULONG,
+    HASH_KEY_CONST_STRING
 } hash_key_enum;
 
 typedef enum
@@ -137,6 +138,7 @@ typedef struct hash_key_t {
     hash_key_enum type;
     union {
         char *str;
+        const char *c_str;
         unsigned long ul;
     };
 } hash_key_t;
@@ -346,7 +348,7 @@ int hash_delete(hash_table_t *table, hash_key_t *key);
  * function. It can be used for any purpose the caller wants. The callback
  * parameter list is:
  *
- * bool callback(hash_entry_t *item, hash_table_t *user_data);
+ * bool callback(hash_entry_t *item, void *user_data);
  *
  * WARNING: Do not modify the contents of the table during an iteration it will
  * produce undefined results. If you need to visit each item in the table and
