@@ -23,6 +23,7 @@
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 #include "simplebuffer.h"
 #include "ref_array.h"
 #include "ini_comment.h"
@@ -246,8 +247,9 @@ static int value_fold(struct simplebuffer *unfolded,
              * Fold if we found the separator or the first line
              * is too long right away
              */
-            else if (((buf[i] == ' ') || (buf[i] == '\t')) ||
-                     ((best_place == 0) && (i == 0))) {
+            else if (((!((best_place > 0) && (idx == 0) && (i == 0))) &&
+                       ((buf[i] == ' ') || (buf[i] == '\t'))) ||
+                       ((best_place == 0) && (i == 0))) {
                 next_place = i;
                 TRACE_INFO_NUMBER("Next place:", next_place);
             }
@@ -320,6 +322,49 @@ static int value_fold(struct simplebuffer *unfolded,
     return error;
 }
 
+static int trim_last(struct value_obj *vo)
+{
+    int error = EOK;
+    uint32_t last = 0;
+    uint32_t len = 0;
+    uint32_t idx = 0;
+    char *ptr = NULL;
+    char *part = NULL;
+
+    TRACE_FLOW_ENTRY();
+
+    last = ref_array_len(vo->raw_lengths);
+    if (last) {
+        last--;
+        ref_array_get(vo->raw_lengths, last, (void *)&len);
+        if (len) {
+            ptr = ref_array_get(vo->raw_lines, last, NULL);
+            if (ptr) {
+                part = *((char **)ptr);
+
+                TRACE_INFO_STRING("Value", part);
+                TRACE_INFO_NUMBER("Length", len);
+
+                idx = len - 1;
+
+                TRACE_INFO_NUMBER("Start index", idx);
+
+                while((idx) && (isspace(part[idx]))) idx--;
+                if (idx != len - 1) {
+                    TRACE_INFO_NUMBER("End index", idx);
+                    len = idx + 1;
+                    error = ref_array_replace(vo->raw_lengths, last, (void *)&len);
+                    if (error) {
+                        TRACE_ERROR_NUMBER("Failed to update length", error);
+                        return error;
+                    }
+                }
+            }
+        }
+    }
+    TRACE_FLOW_EXIT();
+    return error;
+}
 
 /* Create value from a referenced array */
 int value_create_from_refarray(struct ref_array *raw_lines,
@@ -362,6 +407,14 @@ int value_create_from_refarray(struct ref_array *raw_lines,
     new_vo->boundary = boundary;
     new_vo->ic = ic;
 
+    /* Last line might have spaces at the end, trim them */
+    error = trim_last(new_vo);
+    if (error) {
+        TRACE_ERROR_NUMBER("Failed to trim last", error);
+        value_destroy(new_vo);
+        return error;
+    }
+
     error = value_unfold(new_vo->raw_lines,
                          new_vo->raw_lengths,
                          &(new_vo->unfolded));
@@ -919,7 +972,7 @@ int value_put_comment(struct value_obj *vo, struct ini_comment *ic)
 
     TRACE_FLOW_ENTRY();
 
-    if ((!vo) || (!ic)) {
+    if (!vo) {
         TRACE_ERROR_NUMBER("Invalid input parameter", EINVAL);
         return EINVAL;
     }
@@ -1011,8 +1064,6 @@ int value_serialize(struct value_obj *vo,
 
 #ifdef HAVE_TRACE
 
-extern void ref_array_debug(struct ref_array *ra, int num);
-
         ref_array_debug(vo->raw_lines, 0);
         ref_array_debug(vo->raw_lengths, 1);
 #endif
